@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2005-2007 Red Hat, Inc. All rights reserved.
  *
  * This file is part of the device-mapper userspace tools.
  *
@@ -670,13 +670,7 @@ static int _parse_message(struct dm_event_daemon_message *msg, char **dso_name,
 }
 
 /*
- * dm_event_get_registered_device
- * @dso_name
- * @device_path
- * @mask
- * @next
- *
- * Returns: 0 if handler found, error (-ENOMEM, -ENOENT) otherwise
+ * Returns 0 if handler found; error (-ENOMEM, -ENOENT) otherwise.
  */
 int dm_event_get_registered_device(struct dm_event_handler *dmevh, int next)
 {
@@ -706,6 +700,7 @@ int dm_event_get_registered_device(struct dm_event_handler *dmevh, int next)
 	}
 
 	dm_task_destroy(dmt);
+	dmt = NULL;
 
 	if (msg.data) {
 		dm_free(msg.data);
@@ -727,10 +722,15 @@ int dm_event_get_registered_device(struct dm_event_handler *dmevh, int next)
 	dm_event_handler_set_dso(dmevh, reply_dso);
 	dm_event_handler_set_event_mask(dmevh, reply_mask);
 
-	if (reply_dso)
+	if (reply_dso) {
 		dm_free(reply_dso);
-	if (reply_uuid)
+		reply_dso = NULL;
+	}
+
+	if (reply_uuid) {
 		dm_free(reply_uuid);
+		reply_uuid = NULL;
+	}
 
 	dmevh->dev_name = dm_strdup(dm_task_get_name(dmt));
 	if (!dmevh->dev_name) {
@@ -759,7 +759,8 @@ int dm_event_get_registered_device(struct dm_event_handler *dmevh, int next)
 	if (reply_uuid)
 		dm_free(reply_uuid);
 	_dm_event_handler_clear_dev_info(dmevh);
-	dm_task_destroy(dmt);
+	if (dmt)
+		dm_task_destroy(dmt);
 	return ret;
 }
 
